import torch
import torch.nn as nn
import torch.optim as optim
from torchvision import datasets, transforms
from torch.utils.data import DataLoader

# 数据预处理
transform = transforms.Compose([
    transforms.ToTensor(),  # 转换为FloatTensor并归一化到[0, 1]
    transforms.Normalize((0.5,), (0.5,))  # 归一化
])

# 加载数据集
train_dataset = datasets.MNIST(root='./data', train=True, download=True, transform=transform)
test_dataset = datasets.MNIST(root='./data', train=False, download=True, transform=transform)

# 创建数据加载器
train_loader = DataLoader(train_dataset, batch_size=64, shuffle=True)
test_loader = DataLoader(test_dataset, batch_size=64, shuffle=False)


# RNN网络搭建
class RNN(nn.Module):
    def __init__(self, input_size, hidden_size, output_size, num_layers=1):
        super(RNN, self).__init__()
        self.hidden_size = hidden_size
        self.num_layers = num_layers
        self.rnn = nn.RNN(input_size, hidden_size, num_layers, batch_first=True)
        self.fc = nn.Linear(hidden_size, output_size)

    def forward(self, x):
        # RNN期望的输入形状是(batch, seq, feature)
        # 这里我们将图像的高度（28）视为序列长度，宽度（28）视为特征大小
        # 但由于RNN通常用于处理序列数据，这里我们实际上是在滥用RNN
        batch_size, seq_len, features = x.size()
        x = x.view(batch_size, seq_len, features)

        # 初始化隐藏状态
        h0 = torch.zeros(self.num_layers, batch_size, self.hidden_size).to(x.device)

        # 前向传播RNN
        out, _ = self.rnn(x, h0)

        # 取最后一个时间步的输出
        out = self.fc(out[:, -1, :])
        return out

    # 实例化RNN模型


input_size = 28  # 每行有28个像素点
hidden_size = 64
output_size = 10
num_layers = 1
model = RNN(input_size, hidden_size, output_size, num_layers)

# 损失函数和学习率设置
criterion = nn.CrossEntropyLoss()
optimizer = optim.Adam(model.parameters(), lr=0.001)


# 模型训练
def train(model, device, train_loader, optimizer, epoch):
    model.train()
    for batch_idx, (data, target) in enumerate(train_loader):
        data, target = data, target#.to(device)
        optimizer.zero_grad()
        output = model(data[:, 0, :])  # 只取第一行作为示例（实际上应该遍历所有行或使用整个图像的不同方式）
        loss = criterion(output, target)
        loss.backward()
        optimizer.step()
        if batch_idx % 100 == 0:
            print(
                f'Train Epoch: {epoch} [{batch_idx * len(data)}/{len(train_loader.dataset)} ({100. * batch_idx / len(train_loader):.0f}%)]\tLoss: {loss.item():.6f}')

        # 注意：这里的训练代码只使用了图像的第一行，为了完整训练应该修改以使用整个图像
# 但由于RNN处理图像的限制，这里仅作为演示

# 由于上述训练方式不是处理图像数据的最佳方式，这里不展示完整的训练过程
# 而是直接跳到测试部分，假设我们已经有
# 假设模型已经在某个地方被训练好了，这里我们直接加载它（如果有的话）
# model.load_state_dict(torch.load('rnn_mnist_model.pth'))
# model.eval()

# 为了演示，我们直接使用未训练的模型，但结果将不准确
model.eval()  # 设置为评估模式


# 测试模型
def test(model, device, test_loader):
    with torch.no_grad():
        correct = 0
        total = 0
        for data, target in test_loader:
            data, target = data, target
            # 由于RNN的输入要求，我们需要重新调整数据形状
            # 但这里为了简化，我们仍然只取第一行（这不是实际做法）
            # 正确的做法应该是找到更适合RNN处理图像的方法，比如先通过CNN提取特征
            outputs = model(data[:, 0, :])  # 仅使用第一行作为示例
            _, predicted = torch.max(outputs.data, 1)
            print(target)
            print(predicted)
            total += target.size(0)
            correct += (predicted == target).sum().item()

        print(f'Accuracy of the network on the 10000 test images: {100 * correct / total}%')



            # 设置设备


device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
# model = model.to(device)

# 运行测试

if __name__ == '__main__':
    train(model, torch.device('cuda:0'), train_loader, optimizer, 10)
    test(model, torch.device('cuda:0') , test_loader)